@@ -1,6 +1,6 @@
-/* common.c - common functions used by daisy-player and eBook-speaker.
+/* common.c - common functions used by daisy-player..
  *
- * Copyright (C)2018 J. Lemmens
+ * Copyright (C)2020 J. Lemmens
  *                                                                    
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -18,7 +18,6 @@
  */
 
 #include "daisy.h"
-
 char sink_info[10][100];
 
 void open_xml_file (misc_t *misc, my_attribute_t *my_attribute,
@@ -28,7 +27,7 @@ void open_xml_file (misc_t *misc, my_attribute_t *my_attribute,
    {
       endwin ();
       printf ("%s\n",
-              gettext ("Please try to play this book with eBook-speaker"));
+              gettext ("Please try to play this book with \"eBook-speaker\""));
       beep ();
       quit_daisy_player (misc, my_attribute, daisy);
       _exit (EXIT_SUCCESS);
@@ -114,8 +113,10 @@ void player_ended ()
 void get_realpath_name (char *dir, char *search_str, char *found)
 {
    int total, n;
-   struct dirent **namelist;
+   struct dirent **namelist;char *basec;
 
+   basec = strdup (search_str);
+   search_str = basename (basec);
    total = scandir ((const char *) dir, &namelist, NULL, alphasort);
    if (total == -1)
    {
@@ -124,24 +125,21 @@ void get_realpath_name (char *dir, char *search_str, char *found)
    } // if
    for (n = 0; n < total; n++)
    {
-      char *path, *basec;
+      char *path;
 
       path = malloc (strlen (dir) + strlen (namelist[n]->d_name) + 2);
       if (dir[strlen (dir) - 1] == '/')
-         sprintf (path, "%s%s", dir, namelist[n]->d_name);
+         sprintf (path, "%s/%s", dir, namelist[n]->d_name);
       else
          sprintf (path, "%s/%s", dir, namelist[n]->d_name);
-      basec = strdup (search_str);
-      if (strcasecmp (namelist[n]->d_name, basename (basec)) == 0)
+      if (strcasecmp (namelist[n]->d_name, search_str) == 0)
       {
          strcpy (found, path);
          free (path);
          free (namelist);
-         free (basec);
          return;
       } // if
       free (path);
-      free (basec);
       if (strcmp (namelist[n]->d_name, ".") == 0 ||
           strcmp (namelist[n]->d_name, "..") == 0)
          continue;
@@ -153,13 +151,13 @@ void get_realpath_name (char *dir, char *search_str, char *found)
          if (dir[strlen (dir) - 1] != '/')
             sprintf (new_dir, "%s/%s", dir, namelist[n]->d_name);
          else
-            sprintf (new_dir, "%s%s", dir, namelist[n]->d_name);
+            sprintf (new_dir, "%s/%s", dir, namelist[n]->d_name);
          get_realpath_name (new_dir, search_str, found);
          free (new_dir);
       } // if
    } // for
    free (namelist);
-} // get_realpath_name
+} // get_realpath_name                                           
 
 char *get_dir_content (misc_t *misc, char *dir_name, char *search_str)
 {
@@ -232,8 +230,11 @@ void kill_player (misc_t *misc)
    unlink (misc->tmp_wav);
 } // kill_player
 
-void skip_right (misc_t *misc, daisy_t *daisy)
+void skip_right (misc_t *misc, daisy_t *daisy,
+                 my_attribute_t *my_attribute)
 {
+   time_t secs;
+
    if (misc->cd_type == CDIO_DISC_MODE_CD_DA)
       return;
    if (misc->playing == -1)
@@ -243,8 +244,34 @@ void skip_right (misc_t *misc, daisy_t *daisy)
    } // if
    misc->current = misc->displaying = misc->playing;
    wmove (misc->screenwin, daisy[misc->current].y, daisy[misc->current].x);
-   wrefresh (misc->screenwin);
-   kill_player (misc);
+   wrefresh (misc->screenwin);\
+   secs = time (NULL) - misc->elapsed_seconds;
+   if (secs + 10 >= daisy[misc->current].duration)
+   {
+      if (misc->just_this_item >= 0 &&
+          misc->playing + 1 <= misc->total_items - 1 &&
+          daisy[misc->playing + 1].level <= misc->level)
+      {
+         beep ();
+         kill_player (misc);
+         misc->player_pid = -2;
+         misc->just_this_item = misc->playing = -1;
+         view_screen (misc, daisy);
+         return;
+      }
+      else
+      {
+         misc->playing = misc->displaying = ++misc->current;
+         if (misc->current >= misc->total_items)
+         {
+            quit_daisy_player (misc, my_attribute, daisy);
+            _exit (EXIT_SUCCESS);
+         } // if
+         go_to_time (misc, daisy, my_attribute, 0);
+         return;
+      } // if
+   } // if
+   go_to_time (misc, daisy, my_attribute, secs + 10);
 } // skip_right
 
 int handle_ncc_html (misc_t *misc, my_attribute_t *my_attribute,
@@ -284,7 +311,7 @@ int handle_ncc_html (misc_t *misc, my_attribute_t *my_attribute,
    {
       endwin ();
       printf ("%s\n",
-              gettext ("Please try to play this book with daisy-player"));
+              gettext ("Please try to play this book with \"eBook-speaker\""));
       beep ();
       quit_daisy_player (misc, my_attribute, daisy);
       _exit (EXIT_SUCCESS);
@@ -399,7 +426,7 @@ daisy_t *create_daisy_struct (misc_t *misc,
    xmlTextReaderPtr ptr;
 
    misc->total_pages = 0;
-   *misc->daisy_version = 0;\
+   *misc->daisy_version = 0;
    *misc->ncc_html = *misc->ncx_name = *misc->opf_name = 0;
    find_index_names (misc);
 
@@ -482,7 +509,7 @@ daisy_t *create_daisy_struct (misc_t *misc,
    {
       endwin ();
       printf ("%s\n",
-              gettext ("Please try to play this book with daisy-player"));
+              gettext ("Please try to play this book with \"eBook-speaker\""));
       beep ();
       quit_daisy_player (misc, my_attribute, daisy);
       _exit (EXIT_SUCCESS);
@@ -561,6 +588,11 @@ void get_attributes (misc_t *misc, my_attribute_t *my_attribute,
              xmlTextReaderGetAttribute (ptr, (const xmlChar *) "clipend"));
    if (strcmp (attr, "(null)"))
       strncpy (my_attribute->clip_end, attr, MAX_STR - 1);
+   snprintf (attr, MAX_STR - 1, "%s", (char *)
+             xmlTextReaderGetAttribute
+                      (ptr, (const xmlChar *) "elapsed_seconds"));
+   if (strcmp (attr, "(null)"))
+      misc->elapsed_seconds = atoi (attr);
    snprintf (attr, MAX_STR - 1, "%s", (char *)
              xmlTextReaderGetAttribute (ptr, (const xmlChar *) "href"));
    if (strcmp (attr, "(null)"))
@@ -651,7 +683,7 @@ void get_attributes (misc_t *misc, my_attribute_t *my_attribute,
              xmlTextReaderGetAttribute (ptr, (const xmlChar *) "seconds"));
    if (strcmp (attr, "(null)"))
    {
-      misc->seconds = atoi (attr);
+      misc->seconds = atoll (attr);
       if (misc->seconds < 0)
          misc->seconds = 0;
    } // if
@@ -663,7 +695,7 @@ void get_attributes (misc_t *misc, my_attribute_t *my_attribute,
            xmlTextReaderGetAttribute (ptr,
                                       (const xmlChar *) "pulseaudio_device"));
    if (strcmp (attr, "(null)"))
-      misc->pulseaudio_device = *attr - '0';
+      misc->pulseaudio_device = atoi (attr);
    snprintf (attr, MAX_STR - 1, "%s", (char *)
         xmlTextReaderGetAttribute (ptr, (const xmlChar *) "ocr_language"));
    if (strcmp (attr, "(null)"))
@@ -832,8 +864,7 @@ void go_to_page_number (misc_t *misc, my_attribute_t *my_attribute,
    if (atoi (pn) == 0 || atoi (pn) > misc->total_pages)
    {
       beep ();
-      pause_resume (misc, my_attribute, daisy);
-      pause_resume (misc, my_attribute, daisy);
+      misc->playing = misc->just_this_item = -1;
       return;
    } // if
 
@@ -852,9 +883,7 @@ void go_to_page_number (misc_t *misc, my_attribute_t *my_attribute,
       {
 // at the start of an item
          misc->displaying = misc->playing = misc->current;
-         open_clips_file (misc, my_attribute, daisy[misc->playing].clips_file,
-                          daisy[misc->playing].clips_anchor);
-         misc->elapsed_seconds = time (NULL);
+         go_to_time (misc, daisy, my_attribute, 0);
          return;
       } // if
    } // for
@@ -867,8 +896,8 @@ void go_to_page_number (misc_t *misc, my_attribute_t *my_attribute,
    } // for
 
 // start searching here
-   open_clips_file (misc, my_attribute, daisy[misc->playing].clips_file,
-                    daisy[misc->playing].clips_anchor);
+   open_smil_file (misc, my_attribute, daisy[misc->playing].smil_file,
+                    daisy[misc->playing].smil_anchor);
    while (1)
    {
       if (misc->current_page_number == atoi (pn))
@@ -883,13 +912,13 @@ void go_to_page_number (misc_t *misc, my_attribute_t *my_attribute,
       {
          misc->current = misc->playing = 0;
          misc->current_page_number = daisy[misc->playing].page_number;
-         open_clips_file (misc, my_attribute, daisy[misc->playing].clips_file,
-                          daisy[misc->playing].clips_anchor);
+         open_smil_file (misc, my_attribute, daisy[misc->playing].smil_file,
+                          daisy[misc->playing].smil_anchor);
          return;
       } // if current_page_number >  pn
-      get_next_clips (misc, my_attribute, daisy);
+      get_next_audio_tag (misc, my_attribute, daisy);
    } // while
-} // go_to_page_number
+} // go_to_page_number                                       
 
 void select_next_output_device (misc_t *misc, daisy_t *daisy)
 {
@@ -914,6 +943,7 @@ void select_next_output_device (misc_t *misc, daisy_t *daisy)
       printf ("Please give the following command:\n");
       printf ("\n   sudo gpasswd -a <login-name> audio\n\n");
       printf ("Logout and login again and you can use daisy-player.\n");
+      remove_tmp_dir (misc);
       _exit (EXIT_FAILURE);
    } // if
 
@@ -929,28 +959,33 @@ void select_next_output_device (misc_t *misc, daisy_t *daisy)
       int total;
       FILE *p;
 
-      if ((p = popen ("LANG=C /usr/bin/pactl list sinks", "r")) == NULL)
+      if ((p = popen ("LANGUAGE=C /usr/bin/pactl list sinks", "r")) == NULL)
       {
          beep ();
          endwin ();
          printf ("%s\n",
       "Be sure the package pulseaudio-utils is installed onto your system.");
+         remove_tmp_dir (misc);
          _exit (EXIT_FAILURE);
       } // if
 
       n = -1;
-      str = NULL;
-      len = 0;
       while (! misc->term_signaled)
       {
+         str = NULL;
+         len = 0;
          if (getline (&str, &len, p) == -1)
             break;
          if (strncasecmp (str, "Sink #", 6) == 0)
          {
             memset (sink_info[++n], ' ', 70);
-            *sink_info[n] = str[6];
+            len = strlen (str) - 6 - 1;
+// allow only up to 3 digits
+            if (len > 3)
+               len = 3;
+            strncpy (sink_info[n], &str[6], len);
             if (current_sink < 0 &&
-                *sink_info[n] - '0' == misc->pulseaudio_device)
+                atoi (sink_info[n]) == misc->pulseaudio_device)
                current_sink = n;
          } // if
 
@@ -960,7 +995,7 @@ void select_next_output_device (misc_t *misc, daisy_t *daisy)
          {
             int x;
 
-            strcpy (sink_info[n] + 2, str + 14);
+            strcpy (sink_info[n] + 4, str + 14);
             for (x = strlen (sink_info[n]) - 1; x < 70; x++)
                sink_info[n][x] = ' ';
             sink_info[n][x] = 0 ;
@@ -968,13 +1003,15 @@ void select_next_output_device (misc_t *misc, daisy_t *daisy)
 */
          if (strcasestr (str, "alsa.card_name"))
          {
-            len = strlen (str + 20);
-            memcpy (sink_info[n] + 2, str + 20, len);
-            memset (sink_info[n] + len - 2, ' ', 55 - len);
+             if (strlen (str) > 19)
+               str += 19;
+            len = strlen (str) + 3;
+            memcpy (sink_info[n] + 4, str, len);
+            memset (sink_info[n] + len, ' ', 53 - len);
          } // if
          if (strcasestr (str, "Mute:"))
          {
-            strcpy (sink_info[n] + 53, strstr (str, "Mute:"));
+            strcpy (sink_info[n] + 53, strcasestr (str, "Mute:"));
             sink_info[n][52] = ' ';
          } // if
          if (strcasestr (str, "Volume:"))
@@ -1016,7 +1053,7 @@ void select_next_output_device (misc_t *misc, daisy_t *daisy)
       switch (wgetch (misc->screenwin))
       {
       case 13:
-         misc->pulseaudio_device = *sink_info[current_sink] - '0';
+         misc->pulseaudio_device = atoi (sink_info[current_sink]);
          view_screen (misc, daisy);
          nodelay (misc->screenwin, TRUE);
          return;
@@ -1031,18 +1068,28 @@ void select_next_output_device (misc_t *misc, daisy_t *daisy)
       case 'm':
          if (fork () == 0)
          {
-            pactl ("set-sink-mute", *sink_info[current_sink] - '0', "toggle");
+            char dev[5];
+
+            reset_term_signal_handlers_after_fork ();
+            sprintf (dev, "%d", atoi (sink_info[current_sink]));
+            (void) pactl ("set-sink-mute", dev, "toggle");
             _exit (EXIT_SUCCESS);
          } // if
          break;
       case 'q':
          view_screen (misc, daisy);
+         if (misc->pulseaudio_device < 0)
+            misc->pulseaudio_device = 0;
          return;
       case 'v':
       case '1':
          if (fork () == 0)
          {
-            pactl ("set-sink-volume", *sink_info[current_sink] - '0', "-5%");
+            char dev[5];
+
+            reset_term_signal_handlers_after_fork ();
+            sprintf (dev, "%d", atoi (sink_info[current_sink]));
+            (void) pactl ("set-sink-volume", dev, "-5%");
             _exit (EXIT_SUCCESS);
          } // if
          break;
@@ -1050,10 +1097,14 @@ void select_next_output_device (misc_t *misc, daisy_t *daisy)
       case '7':
          if (fork () == 0)
          {
-            pactl ("set-sink-volume", *sink_info[current_sink] - '0', "+5%");
+            char dev[5];
+
+            reset_term_signal_handlers_after_fork ();
+            sprintf (dev, "%d", atoi (sink_info[current_sink]));
+            (void) pactl ("set-sink-volume", dev, "+5%");
             _exit (EXIT_SUCCESS);
          } // if
-         break;                      
+         break;
       default:
          beep ();
          break;
@@ -1077,7 +1128,7 @@ void reset_term_signal_handlers_after_fork (void)
    signal (SIGINT, SIG_DFL);
    signal (SIGTERM, SIG_DFL);
    signal (SIGHUP, SIG_DFL);
-} // reset_term_signal_handlers
+} // reset_term_signal_handlers_after_fork
 
 void free_all (misc_t *misc, my_attribute_t *my_attribute, daisy_t *daisy)
 {
@@ -1097,8 +1148,8 @@ void free_all (misc_t *misc, my_attribute_t *my_attribute, daisy_t *daisy)
    {
       free (daisy[misc->current].xml_file);
       free (daisy[misc->current].xml_anchor);
-      free (daisy[misc->current].clips_file);
-      free (daisy[misc->current].clips_anchor);
+      free (daisy[misc->current].smil_file);
+      free (daisy[misc->current].smil_anchor);
    } // for
    delwin (misc->titlewin);
    delwin (misc->screenwin);
